@@ -15,7 +15,7 @@ from peewee import (AutoField, BigAutoField, BigIntegerField, CharField,
 from sdssdb.exceptions import SdssdbUserWarning
 
 from .. import BaseModel
-from . import database  # noqa
+from . import database
 
 
 class CatalogdbModel(BaseModel):
@@ -114,18 +114,6 @@ class Gaia_DR2_Clean(CatalogdbModel):
         table_name = 'gaia_dr2_clean'
 
 
-class GalacticGenesis(CatalogdbModel):
-
-    class Meta:
-        table_name = 'galactic_genesis'
-
-
-class GalacticGenesisBig(CatalogdbModel):
-
-    class Meta:
-        table_name = 'galactic_genesis_big'
-
-
 class GUVCat(CatalogdbModel):
 
     objid = BigIntegerField(primary_key=True)
@@ -150,65 +138,59 @@ class SDSS_DR13_PhotoObj(CatalogdbModel):
         table_name = 'sdss_dr13_photoobj'
 
 
-class SDSS_DR14_APOGEE_Visit(CatalogdbModel):
+class SDSS_DR16_APOGEE_Visit(CatalogdbModel):
 
     visit_id = TextField(primary_key=True)
 
     class Meta:
-        table_name = 'sdss_dr14_apogeevisit'
+        table_name = 'sdss_dr16_apogeevisit'
 
 
-class SDSS_DR14_APOGEE_Star(CatalogdbModel):
+class SDSS_DR16_APOGEE_Star(CatalogdbModel):
 
     apstar_id = TextField(primary_key=True)
     apogee_id = TextField()
 
-    visits = ManyToManyField(SDSS_DR14_APOGEE_Visit,
+    gaia = ForeignKeyField(Gaia_DR2,
+                           field='source_id',
+                           column_name='gaia_source_id',
+                           object_id_name='gaia_source_id',
+                           backref='apogee_star')
+
+    visits = ManyToManyField(SDSS_DR16_APOGEE_Visit,
                              through_model=_APOGEE_Star_Visit_Deferred,
                              backref='stars')
 
     class Meta:
-        table_name = 'sdss_dr14_apogeestar'
+        table_name = 'sdss_dr16_apogeestar'
 
 
-class SDSS_DR14_APOGEE_Star_Visit(CatalogdbModel):
+class SDSS_DR16_APOGEE_Star_Visit(CatalogdbModel):
 
-    apstar = ForeignKeyField(SDSS_DR14_APOGEE_Star,
+    apstar = ForeignKeyField(SDSS_DR16_APOGEE_Star,
                              backref='+',
                              lazy_load=False)
 
-    visit = ForeignKeyField(SDSS_DR14_APOGEE_Visit,
+    visit = ForeignKeyField(SDSS_DR16_APOGEE_Visit,
                             backref='+',
                             lazy_load=False)
 
     class Meta:
-        table_name = 'sdss_dr14_apogeestarvisit'
+        table_name = 'sdss_dr16_apogeestarvisit'
 
 
-class SDSS_DR14_ASCAP_Star(CatalogdbModel):
+class SDSS_DR16_APOGEE_Star_AllVisit(CatalogdbModel):
 
-    apstar_id = TextField(primary_key=True)
-    apstar = ForeignKeyField(SDSS_DR14_APOGEE_Star,
-                             backref='ascap_stars')
+    apstar = ForeignKeyField(SDSS_DR16_APOGEE_Star,
+                             backref='+',
+                             lazy_load=False)
+
+    visit = ForeignKeyField(SDSS_DR16_APOGEE_Visit,
+                            backref='+',
+                            lazy_load=False)
 
     class Meta:
-        table_name = 'sdss_dr14_ascapstar'
-
-
-class SDSS_DR14_Cannon_Star(CatalogdbModel):
-
-    cannon_id = TextField(primary_key=True)
-
-    @property
-    def apstars(self):
-        """Returns the associated stars in ``SDSSDR14APOGEEStar``."""
-
-        return (SDSS_DR14_APOGEE_Star
-                .select()
-                .where(SDSS_DR14_APOGEE_Star.apogee_id == self.apogee_id))
-
-    class Meta:
-        table_name = 'sdss_dr14_cannonstar'
+        table_name = 'sdss_dr16_apogeestarallvisit'
 
 
 class SDSS_APOGEE_AllStarMerge_r13(CatalogdbModel):
@@ -219,9 +201,9 @@ class SDSS_APOGEE_AllStarMerge_r13(CatalogdbModel):
     def apstars(self):
         """Returns the stars on `.SDSS_DR14_APOGEE_Star` with matching ``apogee_id``."""
 
-        return (SDSS_DR14_APOGEE_Star
+        return (SDSS_DR16_APOGEE_Star
                 .select()
-                .where(SDSS_DR14_APOGEE_Star.apogee_id == self.apogee_id))
+                .where(SDSS_DR16_APOGEE_Star.apogee_id == self.apogee_id))
 
     class Meta:
         table_name = 'sdss_apogeeallstarmerge_r13'
@@ -240,39 +222,6 @@ class SDSS_DR14_SpecObj(CatalogdbModel):
         table_name = 'sdss_dr14_specobj'
 
 
-class SDSS_DR16_SpecObj(SDSS_DR14_SpecObj):
-
-    specobjid = BigIntegerField(primary_key=True)
-
-    photoobj = ForeignKeyField(SDSS_DR13_PhotoObj,
-                               column_name='bestobjid',
-                               object_id_name='bestobjid',
-                               backref='specobj_dr16')
-
-    class Meta:
-        table_name = 'sdss_dr16_specobj'
-
-
-class Gaia_DR2_TwoMass_Best_Neighbour(CatalogdbModel):
-
-    source_id = BigIntegerField(primary_key=True)
-
-    gaia = ForeignKeyField(Gaia_DR2,
-                           field='source_id',
-                           column_name='source_id',
-                           backref='+',
-                           lazy_load=False)
-
-    twomass = ForeignKeyField(TwoMassPSC,
-                              field='pts_key',
-                              column_name='tmass_pts_key',
-                              backref='+',
-                              lazy_load=False)
-
-    class Meta:
-        table_name = 'gaiadr2_tmass_best_neighbour'
-
-
 class SDSS_DR14_QSO(CatalogdbModel):
 
     pk = BigIntegerField(primary_key=True)
@@ -297,6 +246,46 @@ class unWISE(CatalogdbModel):
         table_name = 'unwise'
 
 
+class Tycho2(CatalogdbModel):
+
+    designation = TextField(primary_key=True)
+
+    class Meta:
+        table_name = 'tycho2'
+
+
+class TIC_v8(CatalogdbModel):
+
+    id = BigIntegerField(primary_key=True)
+
+    tycho2 = ForeignKeyField(Tycho2, field='designation',
+                             column_name='tyc', object_id_name='tyc',
+                             backref='tic')
+
+    twomass_psc = ForeignKeyField(TwoMassPSC, field='designation',
+                                  column_name='twomass_psc',
+                                  backref='tic')
+
+    photoobj = ForeignKeyField(SDSS_DR13_PhotoObj, field='objid',
+                               column_name='sdss', object_id_name='sdss',
+                               backref='tic')
+
+    gaia = ForeignKeyField(Gaia_DR2, field='source_id',
+                           column_name='gaia_int', object_id_name='gaia_int',
+                           backref='tic')
+
+    allwise_target = ForeignKeyField(AllWise, field='designation',
+                                     column_name='allwise', object_id_name='allwise',
+                                     backref='tic')
+
+    kepler_input = ForeignKeyField(KeplerInput_DR10, field='kic_kepler_id',
+                                   column_name='kic', object_id_name='kic',
+                                   backref='tic')
+
+    class Meta:
+        table_name = 'tic_v8'
+
+
 class Legacy_Survey_DR8(CatalogdbModel):
 
     ls_id = BigIntegerField(primary_key=True)
@@ -308,30 +297,23 @@ class Legacy_Survey_DR8(CatalogdbModel):
                            object_id_name='gaia_sourceid',
                            backref='legacy_survey')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_sourceid',
+                          object_id_name='gaia_sourceid',
+                          backref='+')
+
     class Meta:
         table_name = 'legacy_survey_dr8'
 
 
-class Gaia_unWISE_AGN(CatalogdbModel):
+class eBOSS_Target_v5(CatalogdbModel):
 
-    gaia_sourceid = BigIntegerField(primary_key=True)
-
-    unwise = ForeignKeyField(unWISE, field='unwise_objid',
-                             column_name='unwise_objid',
-                             object_id_name='unwise_objid',
-                             backref='+')
-
-    gaia = ForeignKeyField(Gaia_DR2, field='source_id',
-                           column_name='gaia_sourceid',
-                           object_id_name='gaia_sourceid_id',
+    sdss = ForeignKeyField(SDSS_DR13_PhotoObj,
+                           column_name='objid_targeting',
+                           object_id_name='objid_targeting',
                            backref='+')
 
-    class Meta:
-        table_name = 'gaia_unwise_agn'
-
-
-class eBOSS_Taarget_v5(CatalogdbModel):
-
     class Meta:
         table_name = 'ebosstarget_v5'
 
@@ -451,6 +433,11 @@ class BHM_Spiders_Generic_Superset(CatalogdbModel):
 
     ls = ForeignKeyField(Legacy_Survey_DR8, field='ls_id', backref='+')
     gaia = ForeignKeyField(Gaia_DR2, object_id_name='gaia_dr2_source_id', backref='+')
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_dr2_source_id',
+                          object_id_name='gaia_dr2_source_id',
+                          backref='+')
 
     class Meta:
         use_reflection = False
@@ -494,32 +481,28 @@ class Gaia_DR2_WD_SDSS(CatalogdbModel):
         table_name = 'gaia_dr2_wd_sdss'
 
 
-class Gaia_DR2_WD(CatalogdbModel):
+class Gaia_unWISE_AGN(CatalogdbModel):
 
-    source_id = BigIntegerField(primary_key=True)
+    gaia_sourceid = BigIntegerField(primary_key=True)
 
-    gaia = ForeignKeyField(Gaia_DR2,
-                           column_name='source_id',
-                           object_id_name='source_id',
-                           backref='wd',
-                           unique=True)
+    unwise = ForeignKeyField(unWISE, field='unwise_objid',
+                             column_name='unwise_objid',
+                             object_id_name='unwise_objid',
+                             backref='+')
 
-    @property
-    def sdss(self):
-        """Returns records from `.Gaia_DR2_WD_SDSS` with matching ``wd``."""
+    gaia = ForeignKeyField(Gaia_DR2, field='source_id',
+                           column_name='gaia_sourceid',
+                           object_id_name='gaia_sourceid',
+                           backref='+')
 
-        return Gaia_DR2_WD_SDSS.select().where(Gaia_DR2_WD_SDSS.wd == self.wd)
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_sourceid',
+                          object_id_name='gaia_sourceid',
+                          backref='+')
 
     class Meta:
-        table_name = 'gaia_dr2_wd'
-
-
-class Tycho2(CatalogdbModel):
-
-    designation = TextField(primary_key=True)
-
-    class Meta:
-        table_name = 'tycho2'
+        table_name = 'gaia_unwise_agn'
 
 
 class GaiaQSO(CatalogdbModel):
@@ -530,36 +513,30 @@ class GaiaQSO(CatalogdbModel):
         table_name = 'gaia_qso'
 
 
-class TIC_v8(CatalogdbModel):
+class Gaia_DR2_WD(CatalogdbModel):
 
-    id = BigIntegerField(primary_key=True)
+    source_id = BigIntegerField(primary_key=True)
 
-    tycho2 = ForeignKeyField(Tycho2, field='designation',
-                             column_name='tyc', object_id_name='tyc',
-                             backref='tic')
+    gaia = ForeignKeyField(Gaia_DR2,
+                           column_name='source_id',
+                           object_id_name='source_id',
+                           backref='wd',
+                           unique=True)
 
-    twomass_psc = ForeignKeyField(TwoMassPSC, field='designation',
-                                  column_name='twomass_psc',
-                                  backref='tic')
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='source_id',
+                          object_id_name='source_id',
+                          backref='+')
 
-    photoobj = ForeignKeyField(SDSS_DR13_PhotoObj, field='objid',
-                               column_name='sdss', object_id_name='sdss',
-                               backref='tic')
+    @property
+    def sdss(self):
+        """Returns records from `.Gaia_DR2_WD_SDSS` with matching ``wd``."""
 
-    gaia = ForeignKeyField(Gaia_DR2, field='source_id',
-                           column_name='gaia_int', object_id_name='gaia_int',
-                           backref='tic')
-
-    allwise_target = ForeignKeyField(AllWise, field='designation',
-                                     column_name='allwise', object_id_name='allwise',
-                                     backref='tic')
-
-    kepler_input = ForeignKeyField(KeplerInput_DR10, field='kic_kepler_id',
-                                   column_name='kic', object_id_name='kic',
-                                   backref='tic')
+        return Gaia_DR2_WD_SDSS.select().where(Gaia_DR2_WD_SDSS.wd == self.wd)
 
     class Meta:
-        table_name = 'tic_v8'
+        table_name = 'gaia_dr2_wd'
 
 
 class CatWISE(CatalogdbModel):
@@ -587,6 +564,12 @@ class Watchdog(CatalogdbModel):
                            object_id_name='gaia_source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_source_id',
+                          object_id_name='gaia_source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'watchdog'
 
@@ -600,6 +583,12 @@ class BlackCAT(CatalogdbModel):
                            object_id_name='gaia_source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_source_id',
+                          object_id_name='gaia_source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'blackcat'
 
@@ -613,6 +602,12 @@ class XRay_Pulsars(CatalogdbModel):
                            object_id_name='gaia_source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_source_id',
+                          object_id_name='gaia_source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'xray_pulsars'
 
@@ -626,6 +621,12 @@ class LowMassXRayBinaries(CatalogdbModel):
                            object_id_name='gaia_source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_source_id',
+                          object_id_name='gaia_source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'lmxb'
 
@@ -639,6 +640,12 @@ class GalacticMillisecondPulsars(CatalogdbModel):
                            object_id_name='gaia_source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_source_id',
+                          object_id_name='gaia_source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'galactic_millisecond_pulsars'
 
@@ -652,6 +659,12 @@ class TransitionalMillisecondPulsars(CatalogdbModel):
                            object_id_name='gaia_source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_source_id',
+                          object_id_name='gaia_source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'transitional_msps'
 
@@ -665,6 +678,12 @@ class ATNF(CatalogdbModel):
                            object_id_name='gaia_source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_source_id',
+                          object_id_name='gaia_source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'atnf'
 
@@ -683,6 +702,12 @@ class SkyMapper_DR1_1(CatalogdbModel):
                            object_id_name='gaia_dr2_id1',
                            backref='skymapper')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_dr2_id1',
+                          object_id_name='gaia_dr2_id1',
+                          backref='+')
+
     @property
     def twomass1(self):
         """Returns the closest 2MASS PSC source, if defined."""
@@ -751,6 +776,12 @@ class SkyMapperGaia(CatalogdbModel):
                            object_id_name='gaia_source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='gaia_source_id',
+                          object_id_name='gaia_source_id',
+                          backref='+')
+
     skymapper = ForeignKeyField(SkyMapper_DR1_1,
                                 column_name='skymapper_object_id',
                                 object_id_name='skymapper_object_id',
@@ -786,18 +817,46 @@ class YSO_Clustering(CatalogdbModel):
                            object_id_name='source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='source_id',
+                          object_id_name='source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'yso_clustering'
 
 
+class MIPSGAL_Extra(CatalogdbModel):
+
+    mipsgal = TextField(primary_key=True)
+
+    gaia = ForeignKeyField(Gaia_DR2,
+                           field='source_id',
+                           column_name='source_id',
+                           object_id_name='source_id',
+                           backref='+')
+
+    class Meta:
+        table_name = 'mipsgal_extra'
+
+
 class MIPSGAL(CatalogdbModel):
 
     mipsgal = TextField(primary_key=True)
 
     twomass = ForeignKeyField(TwoMassPSC,
+                              field='designation',
+                              column_name='twomass_name',
                               object_id_name='twomass_name',
                               backref='+')
 
+    extra = ForeignKeyField(MIPSGAL_Extra,
+                            field='mipsgal',
+                            column_name='mipsgal',
+                            object_id_name='mipsgal',
+                            backref='original')
+
     @property
     def glimpse(self):
         """One-to-many for GLIMPSE targets."""
@@ -838,6 +897,12 @@ class GeometricDistances_Gaia_DR2(CatalogdbModel):
                            object_id_name='source_id',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='source_id',
+                          object_id_name='source_id',
+                          backref='+')
+
     class Meta:
         table_name = 'geometric_distances_gaia_dr2'
 
@@ -851,6 +916,12 @@ class BHM_RM_v0(CatalogdbModel):
                            object_id_name='source_id_gaia',
                            backref='+')
 
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='source_id_gaia',
+                          object_id_name='source_id_gaia',
+                          backref='+')
+
     unwise = ForeignKeyField(unWISE,
                              column_name='objid_unwise',
                              object_id_name='objid_unwise',
@@ -865,8 +936,105 @@ class BHM_RM_v0(CatalogdbModel):
         table_name = 'bhm_rm_v0'
 
 
+class BHM_RM_v0_2(BHM_RM_v0):
+    pass
+
+
+class Gaia_DR2_RUWE(CatalogdbModel):
+
+    gaia = ForeignKeyField(Gaia_DR2,
+                           column_name='source_id',
+                           object_id_name='source_id',
+                           backref='ruwe')
+
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='source_id',
+                          object_id_name='source_id',
+                          backref='+')
+
+    ruwe = FloatField()
+
+    class Meta:
+        table_name = 'gaia_dr2_ruwe'
+
+
+class SDSS_DR13_PhotoObj_Primary(CatalogdbModel):
+
+    objid = BigIntegerField(primary_key=True)
+
+    photoObj = ForeignKeyField(SDSS_DR13_PhotoObj,
+                               field='objid',
+                               column_name='objid',
+                               object_id_name='objid',
+                               backref='+')
+
+    tic = ForeignKeyField(TIC_v8,
+                          field='sdss',
+                          column_name='objid',
+                          object_id_name='objid',
+                          backref='+')
+
+    class Meta:
+        table_name = 'sdss_dr13_photoobj_primary'
+
+
+class SDSS_DR16_SpecObj(SDSS_DR14_SpecObj):
+
+    specobjid = BigIntegerField(primary_key=True)
+
+    photoobj = ForeignKeyField(SDSS_DR13_PhotoObj,
+                               column_name='bestobjid',
+                               object_id_name='bestobjid',
+                               backref='specobj_dr16')
+
+    photoobj_primary = ForeignKeyField(SDSS_DR13_PhotoObj_Primary,
+                                       column_name='bestobjid',
+                                       object_id_name='bestobjid',
+                                       backref='+')
+
+    class Meta:
+        table_name = 'sdss_dr16_specobj'
+
+
+class Gaia_DR2_TwoMass_Best_Neighbour(CatalogdbModel):
+
+    source_id = BigIntegerField(primary_key=True)
+
+    gaia = ForeignKeyField(Gaia_DR2,
+                           field='source_id',
+                           column_name='source_id',
+                           backref='+',
+                           lazy_load=False)
+
+    tic = ForeignKeyField(TIC_v8,
+                          field='gaia_int',
+                          column_name='source_id',
+                          backref='+')
+
+    twomass = ForeignKeyField(TwoMassPSC,
+                              field='pts_key',
+                              column_name='tmass_pts_key',
+                              backref='+',
+                              lazy_load=False)
+
+    class Meta:
+        table_name = 'gaiadr2_tmass_best_neighbour'
+
+
+class GAIA_ASSAS_SN_Cepheids(CatalogdbModel):
+
+    source_id = BigIntegerField(primary_key=True)
+
+    gaia = ForeignKeyField(Gaia_DR2,
+                           field='source_id',
+                           column_name='source_id',
+                           object_id_name='source_id',
+                           backref='assas')
+
+
 _Gaia_DR2_TwoMass_Deferred.set_model(Gaia_DR2_TwoMass_Best_Neighbour)
-_APOGEE_Star_Visit_Deferred.set_model(SDSS_DR14_APOGEE_Star_Visit)
+_APOGEE_Star_Visit_Deferred.set_model(SDSS_DR16_APOGEE_Star_AllVisit)
 
 
 # Add relational tables to namespace.
@@ -914,4 +1082,11 @@ for rtname in all_tables:
                                                 through_model=RelationalModel,
                                                 backref='+'))
 
+        if tname == 'sdss_dr13_photoobj_primary':
+            RelationalModel._meta.add_field('sdss_dr13_photoobj',
+                                            ForeignKeyField(SDSS_DR13_PhotoObj,
+                                                            column_name='target_id',
+                                                            field='objid',
+                                                            backref='+'))
+
         globals()[model_name] = RelationalModel
